@@ -329,7 +329,7 @@ _gtk_theming_engine_set_context (GtkThemingEngine *engine,
 }
 
 /**
- * gtk_theming_engine_register_property:
+ * gtk_theming_engine_register_property: (skip)
  * @name_space: namespace for the property name
  * @parse_func: parsing function to use, or %NULL
  * @pspec: the #GParamSpec for the new property
@@ -473,7 +473,7 @@ gtk_theming_engine_get (GtkThemingEngine *engine,
  * gtk_theming_engine_get_style_property:
  * @engine: a #GtkThemingEngine
  * @property_name: the name of the widget style property
- * @value: (out) (transfer full): Return location for the property value, free with
+ * @value: Return location for the property value, free with
  *         g_value_unset() after use.
  *
  * Gets the value for a widget style property.
@@ -809,7 +809,7 @@ gtk_theming_engine_get_border_color (GtkThemingEngine *engine,
  * gtk_theming_engine_get_border:
  * @engine: a #GtkthemingEngine
  * @state: state to retrieve the border for
- * @color: (out): return value for the border settings
+ * @border: (out): return value for the border settings
  *
  * Gets the border for a given state as a #GtkBorder.
  *
@@ -881,8 +881,9 @@ gtk_theming_engine_get_margin (GtkThemingEngine *engine,
  *
  * Returns the font description for a given state.
  *
- * Returns: the #PangoFontDescription for the given state. This
- *          object is owned by GTK+ and should not be freed.
+ * Returns: (transfer none): the #PangoFontDescription for the given
+ *          state. This object is owned by GTK+ and should not be
+ *          freed.
  *
  * Since: 3.0
  **/
@@ -1029,7 +1030,7 @@ gtk_theming_engine_load (const gchar *name)
  *
  * Returns the #GdkScreen to which @engine currently rendering to.
  *
- * Returns: a #GdkScreen, or %NULL.
+ * Returns: (transfer none): a #GdkScreen, or %NULL.
  **/
 GdkScreen *
 gtk_theming_engine_get_screen (GtkThemingEngine *engine)
@@ -1183,10 +1184,8 @@ gtk_theming_engine_render_option (GtkThemingEngine *engine,
   gint exterior_size, interior_size, pad, thickness, border_width;
   GtkBorderStyle border_style;
   GtkBorder *border;
-  gdouble radius;
 
   flags = gtk_theming_engine_get_state (engine);
-  radius = MIN (width, height) / 2 - 0.5;
 
   cairo_save (cr);
 
@@ -1211,11 +1210,13 @@ gtk_theming_engine_render_option (GtkThemingEngine *engine,
   if (border_style == GTK_BORDER_STYLE_SOLID)
     {
       cairo_set_line_width (cr, border_width);
+
+      cairo_new_sub_path (cr);
       cairo_arc (cr,
-		 x + exterior_size / 2.,
-		 y + exterior_size / 2.,
-		 (exterior_size - 1) / 2.,
-		 0, 2 * G_PI);
+                 x + exterior_size / 2.,
+                 y + exterior_size / 2.,
+                 (exterior_size - 1) / 2.,
+                 0, 2 * G_PI);
 
       gdk_cairo_set_source_rgba (cr, bg_color);
       cairo_fill_preserve (cr);
@@ -1249,10 +1250,10 @@ gtk_theming_engine_render_option (GtkThemingEngine *engine,
       line_thickness = MAX (1, (3 + interior_size * 2) / 7);
 
       cairo_rectangle (cr,
-		       x + pad,
-		       y + pad + (interior_size - line_thickness) / 2.,
-		       interior_size,
-		       line_thickness);
+                       x + pad,
+                       y + pad + (interior_size - line_thickness) / 2.,
+                       interior_size,
+                       line_thickness);
       cairo_fill (cr);
     }
   if (flags & GTK_STATE_FLAG_ACTIVE)
@@ -1261,16 +1262,17 @@ gtk_theming_engine_render_option (GtkThemingEngine *engine,
       interior_size = MAX (1, exterior_size - 2 * pad);
 
       if (interior_size < 5)
-	{
-	  interior_size = 7;
-	  pad = MAX (0, (exterior_size - interior_size) / 2);
-	}
+        {
+          interior_size = 7;
+          pad = MAX (0, (exterior_size - interior_size) / 2);
+        }
 
+      cairo_new_sub_path (cr);
       cairo_arc (cr,
-		 x + pad + interior_size / 2.,
-		 y + pad + interior_size / 2.,
-		 interior_size / 2.,
-		 0, 2 * G_PI);
+                 x + pad + interior_size / 2.,
+                 y + pad + interior_size / 2.,
+                 interior_size / 2.,
+                 0, 2 * G_PI);
       cairo_fill (cr);
     }
 
@@ -1728,15 +1730,6 @@ gtk_theming_engine_render_background (GtkThemingEngine *engine,
 
   junction = gtk_theming_engine_get_junction_sides (engine);
 
-  if (gtk_theming_engine_has_class (engine, "spinbutton") &&
-      gtk_theming_engine_has_class (engine, "button"))
-    {
-      x += 2;
-      y += 2;
-      width -= 4;
-      height -= 4;
-    }
-
   flags = gtk_theming_engine_get_state (engine);
   gtk_theming_engine_get (engine, flags,
                           "border-width", &border,
@@ -2029,10 +2022,20 @@ gtk_theming_engine_render_expander (GtkThemingEngine *engine,
   if (!running)
     progress = (flags & GTK_STATE_FLAG_ACTIVE) ? 1 : 0;
 
-  if (is_rtl)
-    angle = (G_PI) - ((G_PI / 2) * progress);
+  if (!gtk_theming_engine_has_class (engine, GTK_STYLE_CLASS_HORIZONTAL))
+    {
+      if (is_rtl)
+        angle = (G_PI) - ((G_PI / 2) * progress);
+      else
+        angle = (G_PI / 2) * progress;
+    }
   else
-    angle = (G_PI / 2) * progress;
+    {
+      if (is_rtl)
+        angle = (G_PI / 2) + ((G_PI / 2) * progress);
+      else
+        angle = (G_PI / 2) - ((G_PI / 2) * progress);
+    }
 
   interp = progress;
 
@@ -2256,7 +2259,6 @@ gtk_theming_engine_render_layout (GtkThemingEngine *engine,
   const PangoMatrix *matrix;
   GdkRGBA *fg_color;
   GtkStateFlags flags;
-  GdkScreen *screen;
   gdouble progress;
   gboolean running;
 
@@ -2267,7 +2269,6 @@ gtk_theming_engine_render_layout (GtkThemingEngine *engine,
                           "color", &fg_color,
                           NULL);
 
-  screen = gtk_theming_engine_get_screen (engine);
   matrix = pango_context_get_matrix (pango_layout_get_context (layout));
 
   running = gtk_theming_engine_state_is_running (engine, GTK_STATE_PRELIGHT, &progress);
@@ -2304,7 +2305,6 @@ gtk_theming_engine_render_layout (GtkThemingEngine *engine,
   if (matrix)
     {
       cairo_matrix_t cairo_matrix;
-      PangoMatrix tmp_matrix;
       PangoRectangle rect;
 
       cairo_matrix_init (&cairo_matrix,
@@ -2316,20 +2316,19 @@ gtk_theming_engine_render_layout (GtkThemingEngine *engine,
       pango_matrix_transform_rectangle (matrix, &rect);
       pango_extents_to_pixels (&rect, NULL);
 
-      tmp_matrix = *matrix;
       cairo_matrix.x0 += x - rect.x;
       cairo_matrix.y0 += y - rect.y;
 
       cairo_set_matrix (cr, &cairo_matrix);
     }
   else
-    cairo_translate (cr, x, y);
+    cairo_move_to (cr, x, y);
 
   if (flags & GTK_STATE_FLAG_INSENSITIVE)
     {
       cairo_save (cr);
       cairo_set_source_rgb (cr, 1, 1, 1);
-      cairo_move_to (cr, 1, 1);
+      cairo_move_to (cr, x + 1, y + 1);
       _gtk_pango_fill_layout (cr, layout);
       cairo_restore (cr);
     }
@@ -2604,10 +2603,9 @@ gtk_theming_engine_render_handle (GtkThemingEngine *engine,
   cairo_rectangle (cr, x, y, width, height);
   cairo_fill (cr);
 
-  if (gtk_theming_engine_has_class (engine, "grip"))
+  if (gtk_theming_engine_has_class (engine, GTK_STYLE_CLASS_GRIP))
     {
       GtkJunctionSides sides;
-      gint skip = -1;
 
       cairo_save (cr);
 
@@ -2636,8 +2634,6 @@ gtk_theming_engine_render_handle (GtkThemingEngine *engine,
             height = width;
           else if (height < width)
             width = height;
-
-          skip = 2;
         }
       else if (sides == GTK_JUNCTION_CORNER_BOTTOMLEFT)
         {
@@ -2649,8 +2645,6 @@ gtk_theming_engine_render_handle (GtkThemingEngine *engine,
             }
           else if (height < width)
             width = height;
-
-          skip = 1;
         }
       else if (sides == GTK_JUNCTION_RIGHT)
         {
@@ -2670,8 +2664,6 @@ gtk_theming_engine_render_handle (GtkThemingEngine *engine,
               x += (width - height);
               width = height;
             }
-
-          skip = 3;
         }
       else if (sides == GTK_JUNCTION_CORNER_BOTTOMRIGHT)
         {
@@ -2686,8 +2678,6 @@ gtk_theming_engine_render_handle (GtkThemingEngine *engine,
               x += (width - height);
               width = height;
             }
-
-          skip = 0;
         }
       else if (sides == GTK_JUNCTION_TOP)
         {
@@ -2743,7 +2733,7 @@ gtk_theming_engine_render_handle (GtkThemingEngine *engine,
               gdk_cairo_set_source_rgba (cr, &darker);
               add_path_line (cr, x, yi, x + width, yi);
               cairo_stroke (cr);
-              yi+= 2;
+              yi += 2;
             }
         }
       else if (sides == GTK_JUNCTION_CORNER_TOPLEFT)
@@ -2869,7 +2859,7 @@ gtk_theming_engine_render_handle (GtkThemingEngine *engine,
 
       cairo_restore (cr);
     }
-  else if (gtk_theming_engine_has_class (engine, "paned"))
+  else if (gtk_theming_engine_has_class (engine, GTK_STYLE_CLASS_PANE_SEPARATOR))
     {
       if (width > height)
         for (xx = x + width / 2 - 15; xx <= x + width / 2 + 15; xx += 5)
@@ -2881,11 +2871,11 @@ gtk_theming_engine_render_handle (GtkThemingEngine *engine,
   else
     {
       for (yy = y; yy < y + height; yy += 3)
-	for (xx = x; xx < x + width; xx += 6)
-	  {
-	    render_dot (cr, &lighter, &darker, xx, yy, 2);
-	    render_dot (cr, &lighter, &darker, xx + 3, yy + 1, 2);
-	  }
+        for (xx = x; xx < x + width; xx += 6)
+          {
+            render_dot (cr, &lighter, &darker, xx, yy, 2);
+            render_dot (cr, &lighter, &darker, xx + 3, yy + 1, 2);
+          }
     }
 
   cairo_restore (cr);
@@ -2912,18 +2902,12 @@ gtk_theming_engine_render_activity (GtkThemingEngine *engine,
       gdouble half;
       gint i;
 
-      num_steps = 0;
-
-      gtk_theming_engine_get_style (engine,
-                                    "num-steps", &num_steps,
-                                    NULL);
+      num_steps = 12;
 
       state = gtk_theming_engine_get_state (engine);
       gtk_theming_engine_get (engine, state,
                               "color", &color,
                               NULL);
-      if (num_steps == 0)
-        num_steps = 12;
 
       if (gtk_theming_engine_state_is_running (engine, GTK_STATE_ACTIVE, &progress))
         step = (guint) (progress * num_steps);
@@ -3010,6 +2994,55 @@ lookup_icon_size (GtkThemingEngine *engine,
   return gtk_icon_size_lookup_for_settings (settings, size, width, height);
 }
 
+/* Kudos to the gnome-panel guys. */
+static void
+colorshift_pixbuf (GdkPixbuf *src,
+                   GdkPixbuf *dest,
+                   gint       shift)
+{
+  gint i, j;
+  gint width, height, has_alpha, src_rowstride, dest_rowstride;
+  guchar *target_pixels;
+  guchar *original_pixels;
+  guchar *pix_src;
+  guchar *pix_dest;
+  int val;
+  guchar r, g, b;
+
+  has_alpha       = gdk_pixbuf_get_has_alpha (src);
+  width           = gdk_pixbuf_get_width (src);
+  height          = gdk_pixbuf_get_height (src);
+  src_rowstride   = gdk_pixbuf_get_rowstride (src);
+  dest_rowstride  = gdk_pixbuf_get_rowstride (dest);
+  original_pixels = gdk_pixbuf_get_pixels (src);
+  target_pixels   = gdk_pixbuf_get_pixels (dest);
+
+  for (i = 0; i < height; i++)
+    {
+      pix_dest = target_pixels   + i * dest_rowstride;
+      pix_src  = original_pixels + i * src_rowstride;
+
+      for (j = 0; j < width; j++)
+        {
+          r = *(pix_src++);
+          g = *(pix_src++);
+          b = *(pix_src++);
+
+          val = r + shift;
+          *(pix_dest++) = CLAMP (val, 0, 255);
+
+          val = g + shift;
+          *(pix_dest++) = CLAMP (val, 0, 255);
+
+          val = b + shift;
+          *(pix_dest++) = CLAMP (val, 0, 255);
+
+          if (has_alpha)
+            *(pix_dest++) = *(pix_src++);
+        }
+    }
+}
+
 static GdkPixbuf *
 gtk_theming_engine_render_icon_pixbuf (GtkThemingEngine    *engine,
                                        const GtkIconSource *source,
@@ -3056,8 +3089,7 @@ gtk_theming_engine_render_icon_pixbuf (GtkThemingEngine    *engine,
       else if (state & GTK_STATE_FLAG_PRELIGHT)
         {
           stated = gdk_pixbuf_copy (scaled);
-          gdk_pixbuf_saturate_and_pixelate (scaled, stated,
-                                            1.2, FALSE);
+          colorshift_pixbuf (scaled, stated, 30);
           g_object_unref (scaled);
         }
       else
