@@ -22,7 +22,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#include <gtk/gtk.h>
+#include <gtk/gtkx.h>
 #include "gail.h"
 #include "gailfactory.h"
 
@@ -71,11 +71,9 @@ static guint focus_notify_handler = 0;
 static guint focus_tracker_id = 0;
 static GQuark quark_focus_object = 0;
 
-GAIL_IMPLEMENT_FACTORY (GAIL_TYPE_OBJECT, GailObject, gail_object, GTK_TYPE_OBJECT)
 GAIL_IMPLEMENT_FACTORY (GAIL_TYPE_WIDGET, GailWidget, gail_widget, GTK_TYPE_WIDGET)
 GAIL_IMPLEMENT_FACTORY (GAIL_TYPE_CONTAINER, GailContainer, gail_container, GTK_TYPE_CONTAINER)
 GAIL_IMPLEMENT_FACTORY (GAIL_TYPE_BUTTON, GailButton, gail_button, GTK_TYPE_BUTTON)
-GAIL_IMPLEMENT_FACTORY (GAIL_TYPE_ITEM, GailItem, gail_item, GTK_TYPE_ITEM)
 GAIL_IMPLEMENT_FACTORY_WITH_FUNC (GAIL_TYPE_MENU_ITEM, GailMenuItem, gail_menu_item, gail_menu_item_new)
 GAIL_IMPLEMENT_FACTORY (GAIL_TYPE_TOGGLE_BUTTON, GailToggleButton, gail_toggle_button, GTK_TYPE_TOGGLE_BUTTON)
 GAIL_IMPLEMENT_FACTORY (GAIL_TYPE_IMAGE, GailImage, gail_image, GTK_TYPE_IMAGE)
@@ -132,13 +130,7 @@ gail_get_accessible_for_widget (GtkWidget *widget,
       gint page_num = -1;
 
       notebook = GTK_NOTEBOOK (widget);
-      /*
-       * Report the currently focused tab rather than the currently selected tab
-       */
-      if (notebook->focus_tab)
-        {
-          page_num = g_list_index (notebook->children, notebook->focus_tab->data);
-        }
+      page_num = gtk_notebook_get_current_page (notebook);
       if (page_num != -1)
         {
           obj = gtk_widget_get_accessible (widget);
@@ -167,7 +159,7 @@ gail_get_accessible_for_widget (GtkWidget *widget,
     }
   else if (GTK_IS_TOGGLE_BUTTON (widget))
     {
-      GtkWidget *other_widget = widget->parent;
+      GtkWidget *other_widget = gtk_widget_get_parent (widget);
       if (GTK_IS_COMBO_BOX (other_widget))
         {
           gail_set_focus_widget (other_widget, widget);
@@ -219,10 +211,15 @@ gail_focus_watcher (GSignalInvocationHint *ihint,
         {
           if (GTK_IS_WINDOW (widget))
             {
+              GtkWidget *focus_widget;
               GtkWindow *window;
+              GtkWindowType type;
 
               window = GTK_WINDOW (widget);
-              if (window->focus_widget)
+              focus_widget = gtk_window_get_focus (window);
+              g_object_get (window, "type", &type, NULL);
+
+              if (focus_widget)
                 {
                   /*
                    * If we already have a potential focus widget set this
@@ -235,15 +232,15 @@ gail_focus_watcher (GSignalInvocationHint *ihint,
                           !focus_before_menu)
                         {
                           void *vp_focus_before_menu = &focus_before_menu;
-                          focus_before_menu = window->focus_widget;
+                          focus_before_menu = focus_widget;
                           g_object_add_weak_pointer (G_OBJECT (focus_before_menu), vp_focus_before_menu);
                         }
 
                       return TRUE;
                     }
-                  widget = window->focus_widget;
+                  widget = focus_widget;
                 }
-              else if (window->type == GTK_WINDOW_POPUP) 
+              else if (type == GTK_WINDOW_POPUP)
                 {
 	          if (GTK_IS_BIN (widget))
 		    {
@@ -253,7 +250,7 @@ gail_focus_watcher (GSignalInvocationHint *ihint,
 			{
 			  if (GTK_IS_MENU_SHELL (child))
 			    {
-			      if (GTK_MENU_SHELL (child)->active_menu_item)
+			      if (gtk_menu_shell_get_selected_item (GTK_MENU_SHELL (child)))
 				{
 				  /*
 				   * We have a menu which has a menu item selected
@@ -311,7 +308,7 @@ gail_focus_watcher (GSignalInvocationHint *ihint,
    * plug will report a focus notification.
    */
   if (GTK_IS_SOCKET (widget) &&
-      GTK_SOCKET (widget)->plug_widget == NULL)
+      gtk_socket_get_plug_window (GTK_SOCKET (widget)) != NULL)
     return TRUE;
   /*
    * The widget may not yet be visible on the screen so we wait until it is.
@@ -352,10 +349,12 @@ gail_finish_select (GtkWidget *widget)
   if (GTK_IS_MENU_ITEM (widget))
     {
       GtkMenuItem* menu_item;
+      GtkWidget *submenu;
 
       menu_item = GTK_MENU_ITEM (widget);
-      if (menu_item->submenu &&
-          !gtk_widget_get_mapped (menu_item->submenu))
+      submenu = gtk_menu_item_get_submenu (menu_item);
+      if (submenu &&
+          !gtk_widget_get_mapped (submenu))
         {
           /*
            * If the submenu is not visble, wait until it is before
@@ -363,7 +362,7 @@ gail_finish_select (GtkWidget *widget)
            */
           gulong handler_id;
 
-          handler_id = g_signal_handler_find (menu_item->submenu,
+          handler_id = g_signal_handler_find (submenu,
                                               G_SIGNAL_MATCH_FUNC,
                                               g_signal_lookup ("map",
                                                                GTK_TYPE_WINDOW),
@@ -372,11 +371,10 @@ gail_finish_select (GtkWidget *widget)
                                               (gpointer) gail_map_submenu_cb,
                                               NULL); 
           if (!handler_id)
-            g_signal_connect (menu_item->submenu, "map",
+            g_signal_connect (submenu, "map",
                               G_CALLBACK (gail_map_submenu_cb),
                               NULL);
             return;
-
         }
       /*
        * If we are waiting to report focus on a menubar or a menu item
@@ -425,8 +423,11 @@ gail_map_submenu_cb (GtkWidget *widget)
 {
   if (GTK_IS_MENU (widget))
     {
-      if (GTK_MENU (widget)->parent_menu_item)
-        gail_finish_select (GTK_MENU (widget)->parent_menu_item);
+      GtkWidget *parent_menu_item;
+
+      parent_menu_item = gtk_menu_get_attach_widget (GTK_MENU (widget));
+      if (parent_menu_item)
+        gail_finish_select (parent_menu_item);
     }
 }
 
@@ -457,12 +458,12 @@ gail_deselect_watcher (GSignalInvocationHint *ihint,
     {
       GtkWidget *parent_menu_shell;
 
-      parent_menu_shell = GTK_MENU_SHELL (menu_shell)->parent_menu_shell;
+      parent_menu_shell = gtk_menu_shell_get_parent_shell (GTK_MENU_SHELL (menu_shell));
       if (parent_menu_shell)
         {
           GtkWidget *active_menu_item;
 
-          active_menu_item = GTK_MENU_SHELL (parent_menu_shell)->active_menu_item;
+          active_menu_item = gtk_menu_shell_get_selected_item (GTK_MENU_SHELL (parent_menu_shell));
           if (active_menu_item)
             {
               gail_focus_notify_when_idle (active_menu_item);
@@ -488,7 +489,6 @@ gail_switch_page_watcher (GSignalInvocationHint *ihint,
 {
   GObject *object;
   GtkWidget *widget;
-  GtkNotebook *notebook;
 
   object = g_value_get_object (param_values + 0);
   g_return_val_if_fail (GTK_IS_WIDGET(object), FALSE);
@@ -498,8 +498,7 @@ gail_switch_page_watcher (GSignalInvocationHint *ihint,
   if (!GTK_IS_NOTEBOOK (widget))
     return TRUE;
 
-  notebook = GTK_NOTEBOOK (widget);
-  if (!notebook->focus_tab)
+  if (gtk_notebook_get_current_page (GTK_NOTEBOOK (widget)) == -1)
     return TRUE;
 
   gail_focus_notify_when_idle (widget);
@@ -666,7 +665,7 @@ gail_deactivate_watcher (GSignalInvocationHint *ihint,
 
   g_return_val_if_fail (GTK_IS_MENU_SHELL(widget), TRUE);
   shell = GTK_MENU_SHELL(widget);
-  if (!shell->parent_menu_shell)
+  if (! gtk_menu_shell_get_parent_shell (shell))
     focus = focus_before_menu;
       
   /*
@@ -702,7 +701,7 @@ gail_focus_tracker_init (void)
        * We cannot be sure that the classes exist so we make sure that they do.
        */
       g_type_class_ref (GTK_TYPE_WIDGET);
-      g_type_class_ref (GTK_TYPE_ITEM);
+      g_type_class_ref (GTK_TYPE_MENU_ITEM);
       g_type_class_ref (GTK_TYPE_MENU_SHELL);
       g_type_class_ref (GTK_TYPE_NOTEBOOK);
 
@@ -719,7 +718,7 @@ gail_focus_tracker_init (void)
        * a menu item in a menu.
        */
       g_signal_add_emission_hook (
-             g_signal_lookup ("select", GTK_TYPE_ITEM), 0,
+             g_signal_lookup ("select", GTK_TYPE_MENU_ITEM), 0,
              gail_select_watcher, NULL, (GDestroyNotify) NULL);
 
       /*
@@ -727,7 +726,7 @@ gail_focus_tracker_init (void)
        * move from a menu item in a menu to the parent menu.
        */
       g_signal_add_emission_hook (
-             g_signal_lookup ("deselect", GTK_TYPE_ITEM), 0,
+             g_signal_lookup ("deselect", GTK_TYPE_MENU_ITEM), 0,
              gail_deselect_watcher, NULL, (GDestroyNotify) NULL);
 
       /*
@@ -877,7 +876,6 @@ gail_accessibility_module_init (void)
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_WIDGET, gail_widget);
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_CONTAINER, gail_container);
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_BUTTON, gail_button);
-  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_ITEM, gail_item);
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_MENU_ITEM, gail_menu_item);
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_TOGGLE_BUTTON, gail_toggle_button);
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_IMAGE, gail_image);
@@ -913,9 +911,6 @@ gail_accessibility_module_init (void)
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_RADIO_MENU_ITEM, gail_radio_menu_item);
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_EXPANDER, gail_expander);
 
-  /* LIBGNOMECANVAS SUPPORT */
-  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_OBJECT, gail_object);
-
   atk_focus_tracker_init (gail_focus_tracker_init);
   focus_tracker_id = atk_add_focus_tracker (gail_focus_tracker);
 
